\name{README.debug}
\alias{README.debug}
\title{How to use the debug package}
\description{For how to use the \pkg{debug} package, see DETAILS.}
\usage{README.debug() # shows this documentation}
\details{
\code{debug} is an alternative to \code{trace} and \code{browser}, offering:

\itemize{
\item a visible code window with line-numbered code and highlighted execution point;
\item the ability to set (conditional) breakpoints in advance, at any line number;
\item the opportunity to keep going after errors;
\item multiple debugging windows open at once (when one debuggee calls another, or itself);
\item full debugging of \code{on.exit} code;
\item the ability to move the execution point around without executing intervening statements;
\item direct interpretation of typed-in statements, as if they were in the function itself.
}

Even if you don't write functions, or even if you don't write buggy functions, you may find it helpful to run the debugger on functions in \code{package:base} or other packages. Watching what actually happens while a function is executing, can be much more informative than staring at a piece of code.

Debugging your function \code{f} is a two-stage process. First, call \code{mtrace(f)} to store debugging information on \code{f}, and to overwrite \code{f} with a debug-ready version that will call the debugger itself. Second, do whatever you normally do at the command prompt to invoke \code{f}. This is often a direct call to'f', but can be any command that eventually results in \code{f} being invoked. [The third, fourth, etc. stages, in which you actually fix the problem, are not covered here!]

When \code{f} is invoked, a window will appear at the bottom of the screen, showing the code of \code{f} with a highlight on the first numbered line. (There is also an asterisk in the far left hand column of the same row, showing that there's a breakpoint.) The command prompt in \R will change to "D(\dots)> ", showing that you are "inside" a function that is being debugged. The debugger is now in "step mode". Anything you type will be evaluated in the frame of the function-- this includes assignments and creation of new variables. If you just type <ENTER>, the highlighted statement in \code{f} will be executed. The result of the statement will be printed in the \R command window, and the highlight will (probably) move in the \code{f} code window.

To progress through the code of \code{f}, you can keep pressing <ENTER>, but you can also type \code{go()} to put the debugger into "go mode", whereupon it will keep executing code statements without manual intervention. In "go mode", nothing will be printed in the command window (except if there are \code{cat} or \code{print} calls in the function code) until either:

\itemize{
\item the function completes normally, or
\item an error occurs in the function, or
\item a breakpoint is triggered.
}

In the first case, control is returned to the normal \R command prompt, just as if the debugger had not been used. In the other cases, the D(\dots)> prompt will return and the line that caused the error (or that has the breakpoint) will be highlighted in the code window. You are then back in step mode. If there was an error, you can type statement(s) that will cause the error not to happen when the highlighted line executes again, or you can move the highlighted execution point to another line number by calling \code{\link{skip}}. Execution carries on quite normally after errors, just as if the offending statement had been wrapped in a \code{try} call. If your function eventually exits normally (i.e. not via \code{qqq()}, as described next), it will be as if the error never happened (though the error message(s) will be displayed when the \R command prompt returns).

When in step mode, you can finish debugging and return to the normal \R command prompt by typing \code{qqq()}. <ESC> also seems to work, but \code{qqq()} is probably safer. If you type <ESC> while in go mode, you will usually be returned to step mode, but sometimes to the \R command prompt, and sometimes there will be no effect (e.g. if C code is running).

Breakpoints, including conditional breakpoints, are set and cleared by \code{\link{bp}}. Calling \code{go(n)} puts the debugger into go mode, but also sets a temporary breakpoint at line \code{n}, which will be triggered the first time execution reaches line \code{n} but not subsequently.

When the main function code has completed, the debugger moves into any \code{on.exit} code, which is also displayed and line-numbered in the code window. (Even if there are no calls to \code{on.exit}, a numbered NULL statement is placed in the exit code section, so that you can always set a "run-until-function-finishes" breakpoint.) If you exit via \code{qqq()}, the exit code will not be executed first; this can lead to subsequent trouble with open connections, screwed-up \code{par} values in graphics, etc.. To make sure the exit code does get executed:

\itemize{
\item use \code{skip} to move to the start of the exit code;
\item then use \code{go(n)} to run to the final NULL in the exit code;
\item then use \code{qqq()} to finish debugging.
}

When you want to restore \code{f} to its normal non-debugging state (and you are back at the real \R command prompt), type \code{mtrace(f,FALSE)}. To restore all debuggees, type \code{mtrace.off()} (qv). It is advisable not to \code{save} functions in an \code{mtrace}d state; to avoid manual untracing and retracing, look up \code{Save} in package \pkg{mvbutils}.

You can debug several functions "at once" (e.g. if \code{f} calls \code{g}, you can \code{mtrace} both \code{f} and \code{g}, with \code{mtrace(g)} called either inside or outside the debugger), causing several code windows to be open simultaneously. If \code{f} is called again inside \code{f} (either via some statement in \code{f}, or from something you type in step mode), another \code{f} code window will open. The number in the window title is the frame number, and the currently-active frame number is shown in the D(\dots)> prompt.

For statements typed in at the D(\dots)> prompt, only the first syntactically-complete \R expression will be executed; thus, typing \code{a <- 1; a <- 2} will set \code{a} to 1, but typing \code{\{ a <- 1; a <- 2\}} will set \code{a} to 2.

S3 methods work fine with \code{mtrace}, but S4 methods don't (probably fixable, but low priority; I have no plans to use them, and hope not to have to debug other people's!).

For further information, see R-news 3/3.
}
\section{Options}{
Command recall is ON by default, but this means that anything typed while debugging will also be seen in \code{history()} after leaving the debugger. If this is a problem, set \code{options( debug.command.recall=FALSE)} (probably in \code{.First}).

In step mode, only objects with \code{object.size} < 8192 bytes will be printed in full by default; for larger objects, a summary is given instead. You can force printing of any individual object via \code{print}, but you can also increase (or decrease) the threshold to \code{X} bytes, by setting \code{options( threshold.debug.autoprint.size=X)}.

Various TCL/TK-related aspects of the code window can be altered:

\itemize{
\item \code{tab.width} defaults to 4, for indenting code lines (not related to TCL/TK)
\item \code{debug.font} defaults to "Courier"; try e.g. ="Courier 24 italic"
\item \code{debug.height} (in lines) defaults to 10
\item \code{debug.width} (in characters) defaults to 120
\item \code{debug.screen.pos} defaults to "+5-5" for BL corner; try "-5-5" for BR, "-5+5" for TR, "+5+5" for TL.
}
}
\note{
\emph{Display bugs}: There have been sporadic and unreproduceable display problems with the TCL/TK window; sometimes the window frame will appear, but with garbled contents or no contents at all. With RTERM in MS-Windows, a couple of ALT-TABs and mouse clicks to change focus are sometimes necessary. In extremis, the window will eventually sort itself out if you manually minimize, maximize, and restore it-- admittedly an irritation. Although the problem seems less frequent these days, I'm not convinced it is fully solved.

If you encounter this problem, try setting \code{options(shakeup.debug.windows=TRUE)} (also in \code{.First}). The price paid for setting this option, is that window focus will switch away from \R to the new code window when the latter is first created, which is mildly irritating. On MS-Windows, I have a fix for that too, but it requires a DLL written in Delphi not C, so I can't CRAN it; let me know if you are desperate.

For ESS users: I'm not an Emacs user and so haven't tried ESS with the \pkg{debug} package myself, but a read-through of the ESS documentation suggests that at least one ESS variable may need changing to get the two working optimally. Please check the ESS documentation for further details on these points (and see also \code{README.mvbutils} in the \pkg{mvbutils} package). I will update this helpfile when I have more info on what works.

\itemize{
\item The prompt will change when the debugger is running, so you may want to change "inferior-ess-prompt". Prompts will always be of the form D(XXX)> where XXX is a positive integer.
\item Command recall probably won't work inside the debugger (or if it does, it's thanks to ESS rather than to R's own command recall mechanism). It should be disabled by default; if you do get error messages about "history not available", make sure to set \code{options( debug.command.recall=FALSE)} before debugging.
}
}
\seealso{ \code{\link{mtrace}}, \code{\link{go}}, \code{\link{skip}}, \code{\link{qqq}}, \code{\link{bp}}, \code{\link{get.retval}}, \code{\link{mtrace.off}}, \code{\link{check.for.tracees}}}
\author{Mark Bravington}
\keyword{debugging}

\eof
\name{bp}
\alias{bp}
\title{Breakpoints for debugging}
\description{Sets/clears breakpoints (including conditional) breakpoints in functions that have been \code{mtrace}d for debugging.}
\usage{bp( line.no, expr=TRUE, fname) # fname rarely needed}
\arguments{
\item{ line.no}{line number}
\item{ expr}{unquoted expression to be tested when execution reaches \code{line.no}}
\item{ fname}{name of function to twiddle breakpoints in}
}
\details{
Breakpoints can only be set after \code{mtrace}ing a function, and are normally set while the function is actually being debugged. The simplest way is to look at the code window to identify which lines to stop at, call \code{bp(n)} for each of those lines, and then call \code{go()} to enter go mode. Your function code will then be executed without pausing for input until a breakpoint is triggered (or an error occurs, or the function finishes normally). To clear a breakpoint for line \code{n}, type \code{bp(n,FALSE)}.

All line-numbered statements actually have an associated breakpoint expression. When the debugger reaches a line-numbered statement, it evaluates the corresponding breakpoint expression in the function's frame. If the result is not identical to \code{FALSE}, the breakpoint is triggered. By default, all statements have their breakpoint expressions set to \code{FALSE} (by \code{mtrace}), except for line 1 where the expression is set to TRUE.

After setting a breakpoint for line \code{n}, you will see an asterisk (*) in line \code{n} of the left-hand column of the code window. The asterisk is shown whenever the unevaluated breakpoint expression is not identical to \code{FALSE}.

Conditional breakpoints are just expressions other than \code{TRUE} or \code{FALSE}. To get the debugger to stop at line 5 whenver \code{a} is greater than \code{b}, type \code{bp( 5, a>b)}-- don't \code{quote()} the breakpoint expression. Any statement, including a braced statement, can be used, and the debugger will only pause if the result is not \code{FALSE}. You can therefore use "non-breaking breakpoints" to patch expressions into the code. For instance, if you realize that you should have inserted the statement \code{a <- a+1} just before line 7 of your code, you can type \code{bp( 7, \{ a <- a+1; FALSE\}); when the debugger reaches line 7, it will increment }a' but will not switch to step mode, because the overall result was \code{FALSE}.

Sometimes it is useful to clear the line 1 breakpoint before invoking a function, especially if the function is being called repeatedly. The debugger actually starts in go mode, and does not display a code window or pause for input until a breakpoint is triggered; so if the line 1 breakpoint is cleared, execution can continue at full speed until an error occurs (or another breakpoint is triggered). To adjust breakpoints before a function is invoked, you will need to use the \code{fname} argument. To set/clear breakpoints in function \code{f} at lines other than 1, first type \code{tracees$f$line.list} to see which line numbers correspond to which statements.

Breakpoints in body code apply "globally" to all incarnations of a function, and will be retained when the debugger finishes and the \R prompt returns. Breakpoint expressions for \code{f} will be saved in \code{tracees$f$breakpoints}.

Breakpoints can be set in \code{on.exit} code as well (but are specific to the incarnation they are set in). It is often useful to set a breakpoint at the first exit code statement (which will be \code{NULL} if \code{on.exit} has not yet been called); this has the effect of a "run-until-finished-then-pause" breakpoint. Whenever \code{on.exit} is called, any existing exit code breakpoints are lost; but if any were present, a new unconditional breakpoint is set at the start of the exit code.

Breakpoints are evaluated in step mode too, but the debugger remains in step mode whatever the result.

At present, all breakpoints are destroyed when functions are edited (though if you use \code{fixr}, \code{mtrace} will be re-applied automatically). However, the S+ versions of \code{debug} and \code{mvbutils} make an effort to preserve breakpoints across edits, and I plan to introduce something similar in \R.
}
\seealso{ \code{\link{mtrace}}, \code{\link{go}}}
\examples{
\dontrun{
mtrace( glm)
glm( 35)
# Once the debugger starts:
bp(7) # unconditional breakpoint at line 7
bp(7,F) # to clear it.
bp(7,x>1) # conditional; will trigger if "x>1" (or if "x>1" causes error)
bp(1,F,"glm") # can be called BEFORE debugging glm;
# prevents debugger from halting at start of function
qqq() # exit debugger
mtrace.off()
}
}
\author{Mark Bravington}
\keyword{debugging}

\eof
\name{debug-internals}
\alias{debug-internals}
\alias{add.numbers}
\alias{addnum.move.to.next.expr}
\alias{backtrack.to.loop}
\alias{check.legality}
\alias{debug.break}
\alias{debug.do.call}
\alias{debug.mvb.subst}
\alias{debug.next}
\alias{debug.on.exit}
\alias{debug.q}
\alias{debug.return}
\alias{debug.retval}
\alias{debug.sys.on.exit}
\alias{dismiss.debug.window}
\alias{enact.command.r}
\alias{.end.incarnation}
\alias{enter.on.exit}
\alias{eval.bp}
\alias{eval.catching.errors}
\alias{evaluator}
\alias{exit.on.exit}
\alias{find.active.control.frame}
\alias{find.debug.HQ}
\alias{interact}
\alias{launch.debug.windows}
\alias{move.to.next.expression}
\alias{next.incarnation}
\alias{print.if.small}
\alias{screen.line}
\alias{set.a.breakpoint}
\alias{set.global.debug.vars}
\alias{setup.debug.admin}
\alias{skipto.debug}
\alias{stop.here}
\alias{.update.debug.window}
\alias{.update.window.with.on.exit}
\title{Functions used internally by debug package}
\description{Not meant to be called directly.}
\usage{
add.numbers(expr, width = options()$width, numbering = TRUE,
 cat.on.exit = FALSE, expr.offset = 0, line.number.offset = 0)
addnum.move.to.next.expr(nlocal = sys.parent(), final)
backtrack.to.loop(expr, i)
check.legality(thing, call.type)
debug.break()
debug.do.call(what, args)
debug.mvb.subst(expr)
debug.next()
debug.on.exit(new.expr, add = FALSE)
debug.q()
debug.return(\dots)
debug.retval()
debug.sys.on.exit()
dismiss.debug.window(win)
enact.command.r(command, frame)
.end.incarnation()
enter.on.exit()
eval.bp(ex, envir)
eval.catching.errors(i, envir)
evaluator(fname)
exit.on.exit()
find.active.control.frame()
find.debug.HQ(create.if.not = TRUE)
interact(nlocal = sys.parent(), input)
launch.debug.windows()
move.to.next.expression(sorted.out = TRUE, nlocal = sys.parent())
next.incarnation(nlocal = sys.parent())
print.if.small(x, \dots)
screen.line(l, nlocal = sys.parent())
set.a.breakpoint(bp.expr, line.no, frame.number = sys.parent())
set.global.debug.vars(\dots)
setup.debug.admin(nlocal = sys.parent())
skipto.debug(nlocal = sys.parent())
stop.here(nlocal = sys.parent())
.update.debug.window(nlocal = sys.parent(), l)
.update.window.with.on.exit(nlocal = sys.parent(), i, l)
}
\author{Mark Bravington}
\keyword{debugging}
\keyword{internal}

\eof
\name{find.from}
\alias{find.from}
\title{Interactive debugging}
\description{\code{find.from( x, y)} returns the environment where \code{x} would be found if \R was trying to find object \code{x} within environment \code{y}.}
\usage{find.from( char.fname, from=mvb.sys.parent(), look.for.generics=TRUE)}
\arguments{
\item{ char.fname}{length 1 character vector giving the name of the object that's being looked for.}
\item{ from}{an environment, function, or number, showing where to start the search from. Defaults to the calling environment (or .GlobalEnv). If \code{from} is a function, its environment is used as the starting point for the search. If \code{from} is a number, sys.frames()[[ from]] is used.}
\item{ look.for.generics}{logical used internally in recursion; don't change this.}
}
\value{either FALSE (if not found) or an environment.}
\details{Used by \code{mtrace}. Doesn't attempt to deal with S4 methods. Doesn't have a \code{mode} argument (unlike \code{find}).}
\examples{find.from( "as.matrix.data.frame") # NULL; implies "package:base"}
\keyword{debugging}

\eof
\name{get.retval}
\alias{get.retval}
\title{Show current return value when debugging}
\description{When debugging a function, \code{get.retval} gives the current return value, i.e. the result of the most recent valid statement executed in the function body, whether typed or in the original code, and excluding calls to \code{go} etc.. Presumably only relevant when in exit code.}
\usage{get.retval()}
\details{To \emph{change} the return value while in exit code, use \code{skip} to move back into the function code, then \code{return(whatever)}.}
\seealso{ \code{\link{skip}}, \code{\link{go}}}
\author{Mark Bravington}
\keyword{debugging}

\eof
\name{go-skip-qqq}
\alias{go-skip-qqq}
\alias{go}
\alias{skip}
\alias{qqq}
\title{Flow control for debugger}
\description{\code{go}, \code{skip} and \code{qqq} ONLY work inside the debugger, i.e. while you are paused at a D(\dots)> prompt during the execution of a function that has been \code{mtrace}d. \code{go} makes the debugger begn executing code continuously, without user intervention; \code{skip(n)} shifts the execution point; \code{qqq()} quits the debugger.}
\usage{
go(line.no) # line.no can be missing
skip(line.no)
qqq()
}
\arguments{\item{ line.no}{a line number, as shown in the code window (see also DETAILS)}}
\details{
\code{go()} without any argument puts the debugger into "go mode", whereby it starts executing function code without pausing for input (see \code{\link{README.debug}}). \code{go(n)} basically means "run continuously until you reach line n". It sets a temporary breakpoint at line \code{n}, which is triggered the first time line \code{n} is reached and then immediately cleared.

\code{skip(n)} moves the execution point (highlighted in the code window) to line \code{n}, without executing any intervening statements. You can \code{skip} forwards and backwards, and between the main function code and the exit code. You can skip in and out of loops and conditionals, except that you can't skip into a \code{for} loop (the execution point will move to the start of the loop instead). Note that skipping backwards does not undo any statements already executed. \code{skip} is useful for circumventing errors, and for ensuring that exit code gets run before calling \code{qqq()}.

\code{qqq()} quits the debugger, closing all code windows, and returns to the command prompt. No further code statements will be executed, which means no exit code either; take care with open files and connections.
}
\note{Previous versions of the debugger used \code{q()} to quit. With hindsight, this seems unwise, so I've changed it. However, calls to \code{q()} while debugging are still intercepted, and a warning is printed.}
\seealso{ \code{\link{README.debug}}, \code{\link{mtrace}}, \code{\link{bp}}}
\author{Mark Bravington}
\keyword{debugging}

\eof
\name{mtrace}
\alias{mtrace}
\alias{mtrace.off}
\alias{check.for.tracees}
\title{Interactive debugging}
\description{\code{mtrace} sets or clears debugging mode for a function; \code{mtrace.off} clears debugging mode for all functions; \code{check.for.tracees} shows which functions are in debugging mode.}
\usage{
mtrace( fname, tracing=TRUE, char.fname, from=mvb.sys.parent())
# Usual: mtrace( fname) or mtrace( fname, tracing=FALSE)
mtrace.off()
check.for.tracees( where=1)
}
\arguments{
\item{ fname}{quoted or unquoted function name}
\item{ tracing}{TRUE to turn tracing on, FALSE to turn it off}
\item{ char.fname}{(rarely used) if your function name is stored in a character object \code{x}, use \code{char.fname=x}}
\item{ from}{where to start looking for \code{fname} (not usually needed)}
\item{ where}{(character or integer) position in search path}
}
\details{
\code{mtrace(myfun)} modifies the body code of \code{myfun}, and also stores debugging information about \code{myfun} in \code{tracees$myfun}; \code{tracees} is stored in the \code{mvb.session.info} environment (consult \code{README.mvbutils} in the \pkg{mvbutils} package), and is visible from the command line. \code{mtrace} does not modify source code (or other) attributes, so \code{myfun} will "look" exactly the same afterwards. \code{mtrace(myfun,FALSE)} restores \code{myfun} to normal.

Because \code{mtrace} modifies function bodies, calling \code{save.image} or \code{save} while functions are still \code{mtrace}d is probably not a good idea-- if the saved image is reloaded in a new \R session, the debugger won't know how to handle the previously \code{mtrace}d functions, and an error message will be given if they are invoked. The \code{Save} and \code{Save.pos} functions in package \pkg{mvbutils} will get round this without your having to manually untrace and retrace functions.

If you do see a "maybe saved before being un-mtraced?" error message when \code{myfun} is invoked, all is not lost; you can restore \code{myfun} to health via \code{mtrace(myfun,F)}, or put it properly into debugging mode via \code{mtrace(myfun)}. \code{mtrace.off} won't work in such cases, because \code{myfun} isn't included in \code{tracees}.

\code{check.for.tracees} checks for functions which have been \code{mtrace}d, but only in one directory. By contrast, \code{names(tracees)} will return all functions that are currently known to be \code{mtrace}d. However, unlike \code{check.for.tracees}, \code{names(tracees)} won't show functions that were saved during a previous \R session in an \code{mtrace}d state.

\code{mtrace.off} will untrace all functions returned by \code{names( tracees)} and/or \code{check.for.tracees( 1)}.

\code{mtrace} puts a breakpoint (see \code{\link{bp}}) at line 1, but clears all other lines.

\code{mtrace} can handle \code{mlocal} functions, but not (yet) \code{do.in.envir} functions-- the latter appear as monolithic statements in the code window. See package \pkg{mvbutils} for more details.

If you use \code{fixr} to edit functions, \code{mtrace} will automatically be re-applied when an updated function file is sourced back in. Otherwise, you'll need to call \code{mtrace} manually after updating a function.

The \code{from} argument is only used in the following situation. Suppose there is a function \code{f} which first defines functions \code{g} and \code{h}, then calls \code{g}. Now suppose you have \code{mtrace}d \code{g} and that \code{g} is running, with the code window displayed. If you want to \code{mtrace(h)}, the problem is that \code{h} is not visible from the frame of \code{g}. To tell \code{mtrace} where to find \code{g}, call \code{mtrace( h, from=sys.parent())}. You can also replace \code{sys.parent()} with the absolute frame number of \code{f}, if \code{f} has been \code{mtrace}d and its code window is visible). \code{mtrace} will then look through the enclosing environments of \code{from} until it finds a definition of \code{h}.
}
\examples{
\dontrun{
mtrace(glm) # turns tracing on
names( tracees) # "glm"
check.for.tracees( "package:base") # "glm"
glm(stupid.args) # voila le debugger
qqq() # back to command prompt
mtrace( glm, FALSE)
mtrace.off() # turns it off for all functions
}
}
\keyword{debugging}

\eof
